%Created by Ryan J. Smith for SpinCore Technologies 2007
%Modified by Song Kui For SpinCore Technologies 2017
%Now this GUI is compatible with MATLAB 2017a in a 64-bit system  

function varargout = spincore_cpmg_gui(varargin)
% spincore_cpmg_gui M-file for spincore_cpmg_gui.fig
%      spincore_cpmg_gui, by itself, creates a new spincore_cpmg_gui or
%      raises the existing
%      singleton*.
%
%      H = spincore_cpmg_gui returns the handle to a new spincore_cpmg_gui
%      or the handle
%      to
%      the existing singleton*.
%
%      spincore_cpmg_gui('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in spincore_cpmg_gui.M with the given input arguments.
%
%      spincore_cpmg_gui('Property','Value',...) creates a new spincore_cpmg_gui or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before spincore_cpmg_gui_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to spincore_cpmg_gui_OpeningFcn via
%      varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help spincore_cpmg_gui

% Last Modified by GUIDE v2.5 17-Jul-2017 09:15:47

if (exist('./EssentialGuiFiles', 'dir') == 7)
addpath('./EssentialGuiFiles/');
end

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @spincore_cpmg_gui_OpeningFcn, ...
                   'gui_OutputFcn',  @spincore_cpmg_gui_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before spincore_cpmg_gui is made visible.
function spincore_cpmg_gui_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to spincore_cpmg_gui (see VARARGIN)

% Choose default command line output for spincore_cpmg_gui
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% Check if the GUI is closing
set(handles.figure1,'CloseRequestFcn','closeGUI');
guidata(hObject, handles);

%Set default values for input parameters
setappdata(0, 'hCpmgGui', gcf);
hCpmgGui = getappdata(0, 'hCpmgGui');

set(handles.dataButton,'enable','off');
set(handles.stopButton,'enable','off');

setappdata(hCpmgGui, 'dataFlag', 0);

set(handles.sfBox,'string','10.8');
set(handles.swBox,'string','100');
set(handles.pulseBox,'string','13');
set(handles.pulseBox1,'string','26');
set(handles.phaseBox,'string','0');
set(handles.transBox,'string','200');
set(handles.tauBox,'string','3000');
set(handles.echopointsBox,'string','1');
set(handles.echoloopsBox,'string','150');
set(handles.repdelayBox,'string','1');
set(handles.numscansBox,'string','1');
set(handles.adcBox,'string','75');
set(handles.filenameBox,'string','CpmgScan01');
set(handles.firCheck,'value',1);
set(handles.blankingbitBox,'String','2');
set(handles.ampBox,'String','1.0');
set(handles.blankDelayBox, 'String', '3000.0');
set(handles.statusBox,'String','Ready');
drawnow;

% --- Outputs from this function are returned to the command line.
function varargout = spincore_cpmg_gui_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in scanButton ("Perform Scan").
function scanButton_Callback(hObject, eventdata, handles)
% hObject    handle to scanButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

hCpmgGui = getappdata(0, 'hCpmgGui');

error = 0;

set(handles.statusBox,'String','Scan in Progress');
drawnow;

%Poll the input parameters
[SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops,...
    repetition_delay, num_scans, adc_freq, fname, bypass_fir,...
    blankingBit, amp, blank_delay, P1_time]...
    = pollInputParameters(handles);

%Store the input parameters to application memory
storeInputParameters(SF, SW, P2_time, P2_phase, P2_after, tau,... 
    echo_points, echo_loops, repetition_delay, num_scans,... 
    adc_freq, fname, bypass_fir, blankingBit, amp, blank_delay, P1_time);

%Check to make sure that input parameters are valid
if (SF < 0 || isnan(SF))
    errordlg('Spectrometer frequency must be a number greater than or equal to 0');
    error = 1;
elseif (SW < 0 || isnan(SW))
    errordlg('Spectral width must be a number greater than or equal to 0');
    error = 1;
elseif (P2_time < 0.065 || isnan(P2_time))
    errordlg('P2_time must be a number greater than or equal to 0.065');
    error = 1;
elseif (P1_time < 0.065 || isnan(P1_time))
    errordlg('P1_time must be a number greater than or equal to 0.065');
    error = 1;
elseif (P2_after < 0.065 || isnan(P2_after))
    errordlg('Ringdown time must be a number greater than or equal to 0.065');
    error = 1;
elseif (repetition_delay < 0 || isnan(repetition_delay))
    errordlg('Repetition delay must be a number greater than or equal to 0');
    error = 1;
elseif (num_scans < 1 || isnan(num_scans))
    errordlg('At least one scan must be performed.');
    error = 1;
elseif (adc_freq < 0 || isnan(adc_freq))
    errordlg('Please enter correct clock time for your board');
    error = 1;
elseif (echo_points+echo_points*echo_loops > 16384 || echo_points+echo_points*echo_loops < 0)
    errordlg('Points per echo multiplied by number of echoes plus points per echo must be less than 16384')
    error = 1;
elseif(echo_points == 0)
    if (floor( ((tau + echo_loops*(P2_time + 2*tau))/1e6) * SW*1e3) > 16384)
        errordlg('Data will not fit in 16384 RAM. Use fewer echoes.');
        error = 1;
    end
elseif (echo_points+1) > floor(SW*tau/1.0e3)
    errordlg('Points per echo must be less than SpectralWidth*Tau/1000');
    error = 1;
elseif(blankingBit>5)
    errordlg('Blanking bit cannot be a number greater than 8');
    error = 1;
elseif(amp < 0.0 || amp > 1.0)
    errordlg('Amplitude Scaling must be between 0.0 and 1.0');
    error = 1;
elseif(blank_delay < 0.000065)
    errordlg('Blanking delay must be at least 0.000065 (65 ns)');
    error = 1;
end

%If all the parameters are valid, begin CPMG Scan
errorReturn = 0;
if error == 0
    set(handles.dataButton,'enable','on');
    set(handles.stopButton,'enable','on');
    drawnow;
    
    errorReturn = CpmgScan(SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops, ... 
    repetition_delay, num_scans, bypass_fir, adc_freq, blankingBit, amp, blank_delay, P1_time);
end
if errorReturn == -1
    errordlg('Error: Board not found');
elseif errorReturn == -2
    errordlg('Error: Initializing board returned error');
elseif errorReturn == -3
    errordlg('Error: Blanking bit');
end
    set(handles.statusBox,'String','Ready');
    drawnow;
       

% --- Executes on button press in dataButton ("Download Data").
function dataButton_Callback(hObject, eventdata, handles)

hCpmgGui = getappdata(0, 'hCpmgGui');

echo_points = str2double(get(handles.echopointsBox, 'string'));
echo_loops = str2double(get(handles.echoloopsBox, 'string'));
P1_time = getappdata(hCpmgGui, 'P1_time');
tau = getappdata(hCpmgGui, 'tau');
SW = getappdata(hCpmgGui, 'SW');

%Calculate the number of points to display


    if echo_points == 0
        num_points = floor( ((tau + echo_loops*(P1_time + 2*tau))/1e6) * SW*1e3);
        if num_points > 16384
            num_points = 16384;
        end
    else
        num_points = echo_points + echo_points*echo_loops;
        if num_points > 16384
            num_points = 16384;
        end
    end


%Retrieve input parameters associated with last scan from application memory

    SF = getappdata(hCpmgGui, 'SF');
    SW = getappdata(hCpmgGui, 'SW');
    P2_time = getappdata(hCpmgGui, 'P2_time');
    P1_time = getappdata(hCpmgGui, 'P1_time');
    P2_phase = getappdata(hCpmgGui, 'P2_phase');
    P2_after = getappdata(hCpmgGui, 'P2_after');
    tau = getappdata(hCpmgGui, 'tau');
    echo_points = getappdata(hCpmgGui, 'echo_points');
    echo_loops = getappdata(hCpmgGui, 'echo_loops');
    repetition_delay = getappdata(hCpmgGui, 'repetition_delay');
    num_scans = getappdata(hCpmgGui, 'num_scans');
    bypass_fir = getappdata(hCpmgGui, 'bypass_fir');
    adc_freq = getappdata(hCpmgGui, 'adc_freq');
    fname = getappdata(hCpmgGui, 'fname');
    blankingBit = getappdata(hCpmgGui, 'blankingBit');
    amp = getappdata(hCpmgGui, 'amp');
    blank_delay = getappdata(hCpmgGui, 'blank_delay');
    clocktime = clock;
    
    magData=zeros(1,num_points);
    
    set(handles.scanButton,'enable','inactive');
    set(handles.dataButton,'enable','inactive');
    set(handles.buttonNewPlot1,'enable','inactive');
    set(handles.radioReal,'enable','inactive');
    set(handles.radioImag,'enable','inactive');
    set(handles.radioMag,'enable','inactive');

    set(handles.statusBox,'String','Retrieving data.');
    drawnow;

%Check status of board and current scan count
    [pbstatus, pbscans] = pbReadStatus();
    pbstatus=double(pbstatus);
    pbscans=double(pbscans);
    
    if (pbstatus < 0 || isnan(pbstatus))
        pbstatus = 0;
    end
    
    if (pbscans < 0 || isnan(pbscans))
        pbscans = 0;
    end

%Indicate to the user how many scans have been completed
    if bitget(pbstatus, 3) == 1
        PerComplete = sprintf('%d',round(100*pbscans/num_scans));
        set(handles.statusBox,'String', ...
            ['The scan in progress is ' PerComplete '% complete. Retrieving data.']);
        drawnow;
    else
        set(handles.statusBox,'String','Scan complete.  Retrieving data.');
        drawnow;
    end


% Here a subdirectory is either created or overwritten, depending on
% whether or not it already exists.  The data is then downloaded and stored
% to this directory.

    if exist('.\LogFiles') == 7
        cd('.\LogFiles');
        
        if exist(fname) ~= 7
            mkdir(cd,fname);
        end
        
        cd(fname);
        addpath('..\..\EssentialGuiFiles');
        [realData,imagData]=pbGetData(num_points, SF, SW, fname,... 
            num_scans, adc_freq, P2_time, P2_after, repetition_delay, P2_phase,...
            bypass_fir, amp, 'hard');
        cd('..\..\');
    else
        cd('..\LogFiles');
        
        if exist(fname) ~=7
        mkdir(cd,fname);
        end
        
        cd(fname);
        addpath('..\..\EssentialGuiFiles');
        [realData,imagData]=pbGetData(num_points, SF, SW, fname,...
            num_scans, adc_freq, P2_time, P2_after, repetition_delay, P2_phase,...
            bypass_fir, amp, 'hard');
        cd('..\..\EssentialGuiFiles');
    end

%If the data downloaded properly, proceed with analysis and creation of log
%files
    status=0;
    if status == 0
    
        set(handles.statusBox,'String','Retrieval complete! Plotting data.');
        drawnow;

%Convert values in data matrices from INT32 to double since MATLAB will not
%perform many necessary operations on INT32        
        realData=double(realData(1:num_points));
        imagData=double(imagData(1:num_points));

%Set the dataFlag to 1.  This indicates that data has been downloaded
        dataFlag = 1;

%Initialize a vector containing the proper time values needed to correctly
%plot the data
        timeValues = (1:num_points)*(1/SW)*(1/1000);
        
%If scan was not a continuous scan, x axis will display number of points
%instead
        if echo_points ~= 0
            timeValues = 1:num_points;
        end

%Store the data magnitude values in the magData vector
        n = 1:num_points;
        magData(n) = sqrt(realData(n).^2+imagData(n).^2);

%Plot the data specified by the user
        if get(handles.radioReal,'Value')==1
            axes(handles.axes1);
            plot(timeValues,realData(1:num_points));
            
            if echo_points == 0
                xlabel('Time (seconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Real Data');
            axis tight;

        elseif get(handles.radioImag,'Value')==1
            axes(handles.axes1);
            plot(timeValues,imagData(1:num_points));
            
            if echo_points == 0
                xlabel('Time (seconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Imaginary Data');
            axis tight;
        
        elseif get(handles.radioMag,'Value')==1
            axes(handles.axes1);
            plot(timeValues,magData(1:num_points));
            
            if echo_points == 0
                xlabel('Time (seconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Magnitude');
            axis tight;
        end

%Save all pertinent data in a MAT-file and place it along with the other
%log files generated by pbGetDataPlus
    if exist('.\LogFiles') == 7
        cd('.\LogFiles');
        cd(fname);
        save(fname,'clocktime', 'SF', 'SW', 'P2_time', 'P1_time','P2_phase', ...
            'P2_after', 'tau', 'echo_points', 'echo_loops', ...
            'repetition_delay', 'num_scans', 'bypass_fir', 'blankingBit', ...
            'amp', 'blank_delay', 'adc_freq', 'realData', 'imagData', 'magData');
        cd('..\..\');
    else
        cd('..\LogFiles');
        cd(fname);
        save(fname,'clocktime', 'SF', 'SW', 'P2_time', 'P1_time','P2_phase', ...
            'P2_after', 'tau', 'echo_points', 'echo_loops', ...
            'repetition_delay', 'num_scans', 'bypass_fir', 'blankingBit', ...
            'amp', 'blank_delay', 'adc_freq', 'realData', 'imagData', 'magData');
    end
                
        set(handles.statusBox,'String','Ready');
        drawnow;  
        
    elseif status ~= 0
        set(handles.statusBox,'String','Error Retrieving Data');
        drawnow;
    end

%Store relevent data in application memory
    setappdata(hCpmgGui,         'realData', realData);
    setappdata(hCpmgGui,         'imagData', imagData);
    setappdata(hCpmgGui,          'magData', magData);
    setappdata(hCpmgGui,       'timeValues', timeValues);
    setappdata(hCpmgGui,         'dataFlag', dataFlag);
    setappdata(hCpmgGui,               'SW', SW);
    setappdata(hCpmgGui,       'num_points', num_points);
    setappdata(hCpmgGui,      'blankingBit', blankingBit);
    setappdata(hCpmgGui,              'amp', amp);
    setappdata(hCpmgGui,      'blank_delay', blank_delay);
    
    set(handles.scanButton,'enable','on');
    set(handles.dataButton,'enable','on');
    set(handles.buttonNewPlot1,'enable','on');
    set(handles.radioReal,'enable','on');
    set(handles.radioImag,'enable','on');
    set(handles.radioMag,'enable','on');

% --- Executes on button press in buttonNewPlot1.
function buttonNewPlot1_Callback(hObject, eventdata, handles)
% hObject    handle to buttonNewPlot1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

hCpmgGui = getappdata(0, 'hCpmgGui');

%This section of code handles the creation of plots in a new window should
%the user select that option after downloading data

realData = getappdata(hCpmgGui, 'realData');
imagData = getappdata(hCpmgGui, 'imagData');
magData = getappdata(hCpmgGui, 'magData');
timeValues = getappdata(hCpmgGui, 'timeValues');
dataFlag = getappdata(hCpmgGui, 'dataFlag');
num_points = getappdata(hCpmgGui, 'num_points');
echo_points = getappdata(hCpmgGui, 'echo_points');

if dataFlag == 0
    set(handles.statusBox,'String','Error: No data acquired');
    drawnow;
elseif dataFlag == 1
    if get(handles.radioReal,'Value')==1
        figure;
        plot(timeValues,realData(1:num_points));      

        if echo_points == 0
            xlabel('Time (milliseconds)');
        else
            xlabel('Points');
        end
        
        ylabel('Amplitude');
        title('Time Domain - Real Data');
        axis tight;
    elseif get(handles.radioImag,'Value')==1
        figure;
        plot(timeValues,imagData(1:num_points));
        
        if echo_points == 0
            xlabel('Time (milliseconds)');
        else
            xlabel('Points');
        end
        
        ylabel('Amplitude');
        title('Time Domain - Imaginary Data');
        axis tight;
    elseif get(handles.radioMag,'Value')==1
        figure;
        plot(timeValues,magData(1:num_points));
        
        if echo_points == 0
            xlabel('Time (milliseconds)');
        else
            xlabel('Points');
        end
        
        ylabel('Amplitude');
        title('Time Domain - Magnitude');
        axis tight;
    end
end

% --------------------------------------------------------------------
function uipanel2_SelectionChangeFcn(hObject, eventdata, handles)
% hObject    handle to uipanel2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

hCpmgGui = getappdata(0, 'hCpmgGui');

realData = getappdata(hCpmgGui, 'realData');
imagData = getappdata(hCpmgGui, 'imagData');
magData = getappdata(hCpmgGui, 'magData');
timeValues = getappdata(hCpmgGui, 'timeValues');
dataFlag = getappdata(hCpmgGui, 'dataFlag');
echo_points = getappdata(hCpmgGui, 'echo_points');

%This section of code handles the redrawing of plots once the user has
%downloaded data from the RadioProcessor and attempts to change plot
%settings

if dataFlag == 0
    set(handles.statusBox,'String','Error: No data acquired yet.');
    drawnow;    

elseif dataFlag == 1

    switch get(hObject,'Tag')   % Get Tag of selected object
    
        case 'radioReal'
            axes(handles.axes1);
            plot(timeValues,realData);
            
            if echo_points == 0
                xlabel('Time (milliseconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Real Data');
            axis tight;
        
        case 'radioImag'
            axes(handles.axes1);
            plot(timeValues,imagData);
            
            if echo_points == 0
                xlabel('Time (milliseconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Imaginary Data');
            axis tight;
        
        case 'radioMag'
            axes(handles.axes1);
            plot(timeValues,magData);
            
            if echo_points == 0
                xlabel('Time (milliseconds)');
            else
                xlabel('Points');
            end
            
            ylabel('Amplitude');
            title('Time Domain - Magnitude');
            axis tight;
    
    end

end

% --------------------------------------------------------------------
function menuFile_Callback(hObject, eventdata, handles)
% hObject    handle to menuFile (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)


% --------------------------------------------------------------------
function menuSave_Callback(hObject, eventdata, handles)
% hObject    handle to menuSave (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%This code gets the current input parameters and saves them to a
%user-specified location

title = 'SpinCore CpmgNMR Log';
fname = get(handles.filenameBox,'String');

logname=strcat(fname,'_Log.mat');

[filename, pathname] = uiputfile( ...
{ '*.mat','MAT-files (*.mat)'}, ...
  'Save as',logname);

% If 'Cancel' was selected then return
if isequal([filename,pathname],[0,0])
    return
else
    % If filename specified, store variables.
    SF = str2double(get(handles.sfBox,'String'));
    SW = str2double(get(handles.swBox,'String'));
    P2_time = str2double(get(handles.pulseBox,'String'));
    P1_time = str2double(get(handles.pulseBox1,'String'));
    P2_phase = str2double(get(handles.phaseBox,'String'));
    P2_after = str2double(get(handles.transBox,'String'));
    tau = str2double(get(handles.tauBox,'String'));
    echo_points = str2double(get(handles.echopointsBox,'String'));
    echo_loops = str2double(get(handles.echoloopsBox,'String'));
    repetition_delay = str2double(get(handles.repdelayBox,'String'));
    num_scans = str2double(get(handles.numscansBox,'String'));
    adc_freq = str2double(get(handles.adcBox,'String'));
    fname = get(handles.filenameBox,'String');
    fir = get(handles.firCheck,'Value');
    blankingBit = get(handles.blankingbitBox,'String');
    amp = get(handles.ampBox,'String');
    blank_delay = get(handles.blankDelayBox,'String');
    clocktime = clock;
    
    % Construct the full path and save
    File = fullfile(pathname,filename);
    save(File,'title','clocktime','SF','SW','P2_time','P2_after', ...
        'P2_phase', 'tau', 'echo_points','echo_loops', ...
        'repetition_delay','num_scans', 'adc_freq','fname','fir',...
        'blankingBit', 'amp', 'blank_delay');
    
    % Update the status bar
    set(handles.statusBox,'string','Settings logged.');
    drawnow;
end

% --------------------------------------------------------------------
function menuLoad_Callback(hObject, eventdata, handles)
% hObject    handle to menuLoad (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

%This code loads input parameters from a previously saved MAT-file

[filename,pathname] = uigetfile('*.mat','Select the MAT-file');

if isequal([filename,pathname],[0,0])
    return
else
    File = fullfile(pathname,filename);
    logFile = load(File);
end

if logFile.title == 'SpinCore CpmgNMR Log'
    set(handles.sfBox,'string',logFile.SF);
    set(handles.swBox,'string',logFile.SW);
    set(handles.pulseBox,'string',logFile.P2_time);
    set(handles.pulseBox1,'string',logFile.P1_time);
    set(handles.transBox,'string',logFile.P2_after);
    set(handles.phaseBox,'string',logFile.P2_phase);
    set(handles.tauBox,'string',logFile.tau);
    set(handles.echopointsBox,'string',logFile.echo_points);
    set(handles.echoloopsBox,'string',logFile.echo_loops);
    set(handles.repdelayBox,'string',logFile.repetition_delay);
    set(handles.numscansBox,'string',logFile.num_scans);
    set(handles.adcBox,'string',logFile.adc_freq);
    set(handles.filenameBox,'string',logFile.fname);
    set(handles.firCheck,'value',logFile.fir);
    set(handles.blankingbitBox,'string',logFile.blankingBit);
    set(handles.ampBox,'string',logFile.amp);
    set(handles.blankDelayBox,'string',logFile.blank_delay);
    set(handles.statusBox,'string','Settings Loaded.');
    drawnow;
else
    set(handles.statusBox,'string','Invalid Log File');
    drawnow;
end

function sfBox_Callback(hObject, eventdata, handles)
function swBox_Callback(hObject, eventdata, handles)
function pulseBox_Callback(hObject, eventdata, handles)
function pulseBox1_Callback(hObject, eventdata, handles)
function transBox_Callback(hObject, eventdata, handles)
function tauBox_Callback(hObject, eventdata, handles)
function echopointsBox_Callback(hObject, eventdata, handles)
function echoloopsBox_Callback(hObject, eventdata, handles)
function adcBox_Callback(hObject, eventdata, handles)
function filenameBox_Callback(hObject, eventdata, handles)
function firCheck_Callback(hObject, eventdata, handles)
function repdelayBox_Callback(hObject, eventdata, handles)
function numscansBox_Callback(hObject, eventdata, handles)
function phaseBox_Callback(hObject, eventdata, handles)
function blankingbitBox_Callback(hObject, eventdata, handles)
function ampBox_Callback(hObject, eventdata, handles)
function blankDelayBox_Callback(hObject, eventdata, handles)

function menuStop_Callback(hObject, eventdata, handles)
pbStop();
set(handles.statusBox,'string','Scan stopped.');
drawnow;

function stopButton_Callback(hObject, eventdata, handles)

pbStop();
set(handles.statusBox,'string','Scan stopped.');
drawnow;

% --- Executes on button press in resButton.
function resButton_Callback(hObject, eventdata, handles)
% hObject    handle to resButton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

error = 0;

[SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops,...
    repetition_delay, num_scans, adc_freq, fname, bypass_fir, blankingBit,...
    amp, blank_delay, P1_time]...
    = pollInputParameters(handles);

num_points = 8192;
    

if (SF < 0 || isnan(SF))
    errordlg('Spectrometer frequency must be a number greater than or equal to 0');
    error = 1;
elseif (SW < 0 || isnan(SW))
    errordlg('Spectral width must be a number greater than or equal to 0');
    error = 1;
elseif (P2_time < 0.065 || isnan(P2_time))
    errordlg('P2_time must be a number greater than or equal to 0.065');
    error = 1;
elseif (P1_time < 0.065 || isnan(P1_time))
    errordlg('P1_time must be a number greater than or equal to 0.065');
    error = 1;
elseif (P2_after < 0.065 || isnan(P2_after))
    errordlg('Ringdown time must be a number greater than or equal to 0.065');
    error = 1;
elseif (repetition_delay < 0 || isnan(repetition_delay))
    errordlg('Repetition delay must be a number greater than or equal to 0');
    error = 1;
elseif (num_scans < 1 || isnan(num_scans))
    errordlg('At least one scan must be performed.');
    error = 1;
elseif (adc_freq < 0 || isnan(adc_freq))
    errordlg('Please enter correct clock time for your board');
    error = 1;
elseif(blankingBit>8)
    errordlg('Blanking bit cannot be a number greater than 8');
    error = 1;
elseif(amp < 0.0 || amp > 1.0)
    errordlg('Amplitude Scaling must be between 0.0 and 1.0');
    error = 1;
elseif(blank_delay < 0.000065)
    errordlg('Blanking delay must be at least 0.000065 (65 ns)');
    error = 1;
end

if error == 0

    if (SW < 50)
        SW = 500;
    end
    
    set(handles.statusBox,'String','Detecting Resonance Frequency');
    drawnow;
    
    SpnmrWithDelay(num_points, SF, SW, P2_time, P2_after, repetition_delay,...
        num_scans, P2_phase, bypass_fir, adc_freq, amp, blankingBit, blank_delay, 'hard');  

    [realRes, imagRes] = pbGetData();
    realRes = double(realRes);
    imagRes = double(imagRes);
    
    for n = 1:num_points
        compRes(n) = realRes(n)+i*imagRes(n);
    end
    
    compResFft = fft(compRes);
    [maxVal,index] = max(compResFft);
    if index > num_points/2
        offset = -1*(num_points-index+1)*SW*(1/num_points)*(1/1000);
    else
        offset = SW*(index-1)*(1/num_points)*(1/1000);
    end
    
    if index > num_points/2
        resSF = SF + offset;
    else
        resSF = SF + offset;
    end  
    set(handles.sfBox,'String',resSF);
    
    set(handles.statusBox,'String','Ready');
    drawnow;
    
end

% --- Executes on button press in t2Button.
function t2Button_Callback(hObject, eventdata, handles)
% hObject    handle to t2Button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.statusBox,'String','Calculating T2 Value');

[SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops,...
    repetition_delay, num_scans, adc_freq, fname, bypass_fir, blankingBit,...
    amp, blank_delay, P1_time]...
    = pollInputParameters(handles);

error = 0;
if (echo_points ~= 1)
    errordlg('Must perform scan using only 1 point per echo to calculate T2');
    error = 1;
end
    
if (error == 0)
    num_points = echo_points + echo_loops*echo_points;

    [realData,imagData] = pbGetData(); %Get data from board

    %Data read from board consists of 2 16k Int32 arrays.  
    %Convert to double and only use relevant points
    realData=double(realData(1:num_points));
    imagData=double(imagData(1:num_points));
    magData=zeros(1,num_points);

    for n = 1:num_points
        magData(n) = sqrt(realData(n).^2+imagData(n).^2);
    end

    %This assumes 1 point per echo with a distance of 2*tau+P1_time between
    %each point (P1_time = 2*P2_time).
    t = 0:num_points-1;
    timeValues = (t*(P1_time+2*tau)*1E-6);

    %To do the exponential fit, first take the natural log of the data and
    %compute a first order polynomial fit (using least-squares error fitting).
    fit = polyfit(timeValues, log(magData),1);

    %the T2 time is equal to the time constant of the exponential (i.e., the
    %negative inverse of the coefficient of the first order polynomial).
    t2 = -1/fit(1);

    str = sprintf('T2 = %f seconds',t2);

    set(handles.statusBox,'string',str);
    drawnow;

    %residuals are calculated but not currently used
    %residuals = magData - exp(fit(1)*timeValues+fit(2));

    figure;

    subplot(2,1,1);
    plot(timeValues,magData);
    hold on;
    plot(timeValues,exp(fit(1)*timeValues+fit(2)),'r');
    title('Plot of Data with Fitted Exponential');
    xlabel('Time (seconds)');
    ylabel('Amplitude');
    legend('Echo Peaks','Fitted Exponential')

    subplot(2,1,2);
    semilogy(timeValues,magData);
    hold on;
    semilogy(timeValues,exp(fit(1)*timeValues+fit(2)),'r');
    title('Semilog Plot of Data with Fitted Exponential');
    xlabel('Time (seconds)');
    ylabel('Amplitude');
    legend('Echo Peaks','Fitted Exponential')
end

function [SF, SW, P2_time, P2_phase, P2_after, tau, echo_points, echo_loops,...
            repetition_delay, num_scans, adc_freq, fname, bypass_fir, blankingBit,...
            amp, blank_delay, P1_time]...
            = pollInputParameters(handles)
    
    SF = str2double(get(handles.sfBox,'String'));
    SW = str2double(get(handles.swBox,'String'));
    P2_time = str2double(get(handles.pulseBox,'String'));
    P1_time = str2double(get(handles.pulseBox1,'String'));
    P2_phase = str2double(get(handles.phaseBox,'String'));
    P2_after = str2double(get(handles.transBox,'String'));
    tau = str2double(get(handles.tauBox,'String'));
    echo_points = str2double(get(handles.echopointsBox,'String'));
    echo_loops = str2double(get(handles.echoloopsBox,'String'));
    repetition_delay = str2double(get(handles.repdelayBox,'String'));
    num_scans = str2double(get(handles.numscansBox,'String'));
    adc_freq = str2double(get(handles.adcBox,'String'));
    fname = get(handles.filenameBox,'String');

    if (get(handles.firCheck,'Value') == get(handles.firCheck,'Max'))
        % Checkbox is checked
        bypass_fir = 1;
    else
        % Checkbox is not checked
        bypass_fir = 0;
    end
    
    blankingBit = str2double(get(handles.blankingbitBox,'String'));
    amp = str2double(get(handles.ampBox,'String'));
    blank_delay = str2double(get(handles.blankDelayBox,'String'));

function storeInputParameters(SF, SW, P2_time, P2_phase, P2_after, tau,... 
            echo_points, echo_loops, repetition_delay, num_scans,... 
            adc_freq, fname, bypass_fir, blankingBit, amp, blank_delay, P1_time)

    hCpmgGui = getappdata(0, 'hCpmgGui');

    setappdata(hCpmgGui, 'SF', SF);
    setappdata(hCpmgGui, 'SW', SW);
    setappdata(hCpmgGui, 'P2_time', P2_time);
    setappdata(hCpmgGui, 'P1_time', P1_time);
    setappdata(hCpmgGui, 'P2_phase', P2_time);
    setappdata(hCpmgGui, 'P2_after', P2_after);
    setappdata(hCpmgGui, 'tau', tau);
    setappdata(hCpmgGui, 'echo_points', echo_points);
    setappdata(hCpmgGui, 'echo_loops', echo_loops);
    setappdata(hCpmgGui, 'repetition_delay', repetition_delay);
    setappdata(hCpmgGui, 'num_scans', num_scans);
    setappdata(hCpmgGui, 'bypass_fir', bypass_fir);
    setappdata(hCpmgGui, 'adc_freq', adc_freq);
    setappdata(hCpmgGui, 'fname', fname);
    setappdata(hCpmgGui, 'blankingBit', blankingBit);
    setappdata(hCpmgGui, 'amp', amp);
    setappdata(hCpmgGui, 'blank_delay', blank_delay);
    
% --- Executes during object creation, after setting all properties.
function sfBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to sfBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function pulseBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to pulseBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function swBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to swBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function transBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to transBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function tauBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to tauBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function echopointsBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to echopointsBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function echoloopsBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to echoloopsBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function adcBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to adcBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function statusBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to statusBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function filenameBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to filenameBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function repdelayBox_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function numscansBox_CreateFcn(hObject, eventdata, handles)
% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function phaseBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to phaseBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc
    set(hObject,'BackgroundColor','white');
else
    set(hObject,'BackgroundColor',get(0,'defaultUicontrolBackgroundColor'));
end


% --- Executes during object creation, after setting all properties.
function blankingbitBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to blankingbitBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes during object creation, after setting all properties.
function ampBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ampBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object creation, after setting all properties.
function blankDelayBox_CreateFcn(hObject, eventdata, handles)
% hObject    handle to blankDelayBox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



% --- Executes during object creation, after setting all properties.
function pulseBox1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to pulseBox1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
