/**
 * singlepulse_nmr.c
 * This program is used to control the RadioProcessor series of boards.
 *
 * SpinCore Technologies, Inc.
 * www.spincore.com
 * $Date: 2009/07/23 15:06:02 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mex.h"
#include "spinapi.h"
#include "CyclopsScan.h"

int
main (int boardNum, int num_points, double SF, double SW, double pulse_time, 
double trans_time, double repetition_delay, int num_scans,
double tx_phase, int bypass_fir, double adc_freq, float amp_val, char *shape, 
int blankingBit, double blanking_delay, int* errorCYCLOPS, double* totalTime)
{
    SCANPARAMS *myScan;
    CYCLOPSPARAMS *cycScan;
    
  int programBoardReturn;
  //These variables are used for the Title Block in Felix
  char *program_type = "cyclops_nmr";
    myScan = (SCANPARAMS *) malloc (sizeof (SCANPARAMS));
    memset ((void *) myScan, 0, sizeof (SCANPARAMS));
    cycScan = (CYCLOPSPARAMS *) malloc (sizeof (CYCLOPSPARAMS));
    memset ((void *) cycScan, 0, sizeof (CYCLOPSPARAMS));
  myScan->verbose = 1;
  myScan->board_num = boardNum;
  myScan->nPoints = num_points;
  myScan->spectrometerFrequency = SF;
  myScan->spectralWidth = SW;
  myScan->pulseTime = pulse_time;
  myScan->transTime = trans_time;
  myScan->repetitionDelay = repetition_delay;
  if(num_scans>1 && num_scans%4!=0)
  {
        printf("Error: To run CYCLOPS, the Number of Scans must be a mulitple of 4 or 1 for single pulse nmr.\n");
        errorCYCLOPS[0] = -2;
        free (myScan);
        return -1;
    }
  myScan->nScans = num_scans;
  myScan->tx_phase = tx_phase;
  myScan->bypass_fir = bypass_fir;
  myScan->adcFrequency = adc_freq;
    if ((strcmp(shape,"Sinc")==0) || (strcmp(shape,"sinc")==0))
    {
        myScan->use_shape = 1;
    }
    else myScan->use_shape = 0; 
  myScan->amplitude = amp_val;
  myScan->enable_tx = 1;
  myScan->enable_rx = 1;
  myScan->blankingEnable = 1;
  myScan->blankingBit = blankingBit;
  myScan->blankingDelay = blanking_delay;
  
  cycScan->add_sub_re_0 = 1;
  cycScan->add_sub_im_0 = 1;
  cycScan->swap_0 = 0;
  cycScan->add_sub_re_1 = 1;
  cycScan->add_sub_im_1 = 0;
  cycScan->swap_1 = 1;
  cycScan->add_sub_re_2 = 0;
  cycScan->add_sub_im_2 = 0;
  cycScan->swap_2 = 0;
  cycScan->add_sub_re_3 = 0;
  cycScan->add_sub_im_3 = 1;
  cycScan->swap_3 = 1;
  ///
    
  //pb_set_debug(1);
  if (verifyArguments (myScan, myScan->verbose) == 0)
    {
        
        if (pb_count_boards () <= 0)
        {
            printf("No RadioProcessor boards were detected in your system.\n");
            errorCYCLOPS[0] = -3;   // There is no board
            return BOARD_NOT_DETECTED;
        }
        
        pb_select_board (myScan->board_num);
        if (configureBoard (myScan) < 0)	//Set board defaults.
            return -1;
        
          
		totalTime[0] = (myScan->nScans)*( 
										myScan->blankingDelay + 		//ms
										myScan->pulseTime/1000.0 + 		//us
										myScan->transTime/1000.0 + 		//us
										myScan->wait_time + 			//ms
										myScan->repetitionDelay*1000.0);	//S
		myScan->total_Time = totalTime[0];
		outputScanParams (myScan);    
        //programBoardReturn =programBoard(myScan,cycScan); //Program the board.
        if ( programBoard(myScan,cycScan)!= 0)	
        {
            errorCYCLOPS[0] = -1;   // The board does not support CYCLOPS
            pb_close();
            return PROGRAMMING_FAILED;
        }
        
        pb_start ();
        pb_close ();
    }
    errorCYCLOPS[0] = 0;
    free (myScan);
    return 0;
}


int
verifyArguments (SCANPARAMS * scanParams, int verbose)
{
  if (pb_count_boards () > 0
      && scanParams->board_num > pb_count_boards () - 1)
    {
      if (verbose)
	printf ("Error: Invalid board number. Use (0-%d).\n",
		pb_count_boards () - 1);
      return -1;
    }

  if (scanParams->nPoints > RAM_SIZE || scanParams->nPoints < 1)
    {
      if (verbose)
	printf ("Error: Maximum number of points is %d.\n", RAM_SIZE);
      return -1;
    }

  if (scanParams->nScans < 1)
    {
      if (verbose)
	printf ("Error: There must be at least one scan.\n");
      return -1;
    }

  if (scanParams->pulseTime < 0.065)
    {
      if (verbose)
	printf ("Error: Pulse time is too small to work with board.\n");
      return -1;
    }

  if (scanParams->transTime < 0.065)
    {
      if (verbose)
	printf ("Error: Transient time is too small to work with board.\n");
      return -1;
    }

  if (scanParams->amplitude < 0.0 || scanParams->amplitude > 1.0)
    {
      if (verbose)
	printf ("Error: Amplitude value out of range.\n");
      return -1;
    }

  return 0;
}

void
outputScanParams (SCANPARAMS * myScan)
{
	if(myScan->nScans > 1)
    	printf ("\nCYCLOPS NMR Experiment \n");
    else
    	printf ("\nSingle Pulse NMR Experiment \n");
    printf ("--------------------------------------\n");
    //printf ("Filename                : %s\n", myScan->outputFilename);
  printf ("Board Number             : %d\n", myScan->board_num);
  printf ("Number of Points         : %d\n", myScan->nPoints);
  printf ("Number of Scans          : %d\n", myScan->nScans);
  printf ("Use shape                : %d\n", myScan->use_shape);
  printf ("Bypass FIR               : %d\n", myScan->bypass_fir);
  printf ("Amplitude                : %.3lf\n", myScan->amplitude);
  printf ("Spectrometer Frequency   : %.4lf MHz\n", myScan->spectrometerFrequency);
  
  printf ("Specified Spectral Width : %.3lf kHz\n", myScan->spectralWidth);
  printf ("Actual Spectral Width    : %.3lf kHz\n", myScan->actualSpectralWidth/1000);
  
  printf ("TX Phase                 : %.3lf degrees\n", myScan->tx_phase);
  printf ("Pulse Time               : %.3lf us\n", myScan->pulseTime);
  printf ("Trans Time               : %.3lf us\n", myScan->transTime);
  printf ("Repetition Delay         : %.3lf s\n", myScan->repetitionDelay);
  printf ("ADC Frequency            : %.3lf MHz\n", myScan->adcFrequency);
  printf ("Enable Transmitter       : %d\n", myScan->enable_tx);
  printf ("Enable Receiver          : %d\n", myScan->enable_rx);
  printf ("Use TTL Blanking         : %d\n", myScan->blankingEnable);
  printf ("Blanking TTL Flag Bits   : %d\n", myScan->blankingBit);
  printf ("Blanking Delay           : %.3lf us\n", (myScan->blankingDelay)*1000.0);
  printf ("Acquisition Time         : %.3lf ms\n", myScan->wait_time);
  printf ("Total Time               : %.3lf ms\n", myScan->total_Time);
  printf ("--------------------------------------\n");
}

void
make_shape_data (float *shape_array, void *arg,
		 void (*shapefnc) (float *, void *))
{
  shapefnc (shape_array, arg);
}

void
shape_sinc (float *shape_data, void *nlobe)
{
  static double pi = 3.1415926;
  int i;
  int lobes = *((int *) nlobe);
  double x;
  double scale = (double) lobes * (2.0 * pi);

  for (i = 0; i < 1024; i++)
    {
      x = (double) (i - 512) * scale / 1024.0;
      shape_data[i] = sin (x) / x;
      if ((x) == 0.0)
	{
	  shape_data[i] = 1.0;
	}
    }
}

int
configureBoard (SCANPARAMS * myScan)
{
  float shape_data[1024];

  double actual_SW;
  double wait_time;
  double spectralWidth_MHZ = myScan->spectralWidth / 1000.0;

  int dec_amount;
  int cmd = 0;
  int num_lobes = 3;
  int numBoards = pb_count_boards();
  
  if(numBoards > 1)
  {
    printf("\n%d boards detected in your system.  Use pb_select_board(...) to specify\n which board to use.\n\n", numBoards);
  }
  if (pb_init ())
    {
      printf ("Error initializing board: %s\n", pb_get_error ());
      return -1;
    }

  pb_set_defaults ();
  pb_core_clock (myScan->adcFrequency);

  pb_overflow (1, 0);		// reset the overflow counters
  pb_scan_count (1);		// reset scan counter


  // Load the shape parameters
  make_shape_data (shape_data, (void *) &num_lobes, shape_sinc);
  pb_dds_load (shape_data, DEVICE_SHAPE);
  pb_set_amp (((myScan->enable_tx) ? myScan->amplitude : 0), 0);

  //
  /// Set acquisition parameters
  ///
  if (myScan->bypass_fir)
    {
      cmd = BYPASS_FIR;
    }

  dec_amount = pb_setup_filters (spectralWidth_MHZ, myScan->nScans, cmd);
  pb_set_num_points (myScan->nPoints);

  if (dec_amount <= 0)
    {
      if (myScan->verbose)
	printf
	  ("\n\nError: Invalid data returned from pb_setup_filters(). Please check your board.\n\n");
      return INVALID_DATA_FROM_BOARD;
    }

  actual_SW = (myScan->adcFrequency * 1e6) / (double) dec_amount;
  myScan->actualSpectralWidth = actual_SW;
  //printf ("Actual Spectral Width   : %f Hz\n", myScan->actualSpectralWidth);

  wait_time = 1000.0 * (((double) myScan->nPoints) / actual_SW);	// time in ms
  myScan->wait_time = wait_time;
  //printf ("Acquisition Time        : %lf ms\n\n", wait_time);
  
  return 0;
}

int
programBoard (SCANPARAMS * myScan, CYCLOPSPARAMS * cycScan)
{
  int i, start, rx_start, tx_start;
  int nLoop;
  int flagBits = 0;
  flagBits = (1<<myScan->blankingBit)&0x0F;
  pb_start_programming (FREQ_REGS);
  pb_set_freq (0);		//Program Frequency Register 0 to 0
  pb_set_freq (myScan->spectrometerFrequency * MHz);	//Register 1
  pb_set_freq (checkUndersampling (myScan, myScan->verbose));	//Register 2
  pb_stop_programming ();


  // control phases for REAL channel
  pb_start_programming (COS_PHASE_REGS);
  pb_set_phase (0.0);
  pb_set_phase (90.0);
  pb_set_phase (180.0);
  pb_set_phase (270.0);
  pb_stop_programming ();

  // control phases for IMAG channel
  pb_start_programming (SIN_PHASE_REGS);
  pb_set_phase (0.0);
  pb_set_phase (90.0);
  pb_set_phase (180.0);
  pb_set_phase (270.0);
  pb_stop_programming ();

  // control phases for output channel
  pb_start_programming (TX_PHASE_REGS);
  pb_set_phase (myScan->tx_phase);
  pb_set_phase (myScan->tx_phase + 90.0);
  pb_set_phase (myScan->tx_phase + 180.0);
  pb_set_phase (myScan->tx_phase + 270.0);
  pb_stop_programming ();

  ///
  /// Specify pulse program
  ///

  pb_start_programming (PULSE_PROGRAM);
  if(myScan->nScans>1)
  {
  	nLoop = myScan->nScans/4;
  
  //If we have the transmitter enabled, we must include the pulse program to generate the RF pulse.
  if (myScan->enable_tx)
    {
      // Reset phase initially, so that the phase of the excitation pulse will be the same for every scan.
      if(pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE, PHASE_RESET,
			   NO_TRIGGER, myScan->use_shape, myScan->amplitude,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0,
			   0x00, CONTINUE, 0, 1.0 * us) < 0)
		{
            if (myScan->verbose)
			printf("Your board does not support CYCLOPS\n\n");
			return -11;
		}
	


/********************  BEGIN SCAN #0  **********************************************/
      //If blanking is enabled, we must add an additional pulse program interval to compesate for the time the power amplifier needs to "warm up" before we can generate the RF pulse.
      if (myScan->blankingEnable)
	{
	  tx_start =
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE,
				 NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0,
				 flagBits, LOOP, nLoop
				 , myScan->blankingDelay * ms);

	  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_ENABLE,
			       NO_PHASE_RESET, NO_TRIGGER, myScan->use_shape,
			       0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, flagBits , CONTINUE, 0,
			       myScan->pulseTime * us);
	}
      else
	{
	  tx_start =
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_ENABLE,
				 NO_PHASE_RESET, NO_TRIGGER,
				 myScan->use_shape, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, LOOP,
				 nLoop, myScan->pulseTime * us);
	}

      // Output nothing for the transient time.
      pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE,
			   NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, CONTINUE,
			   0, myScan->transTime * us);
    }

  //If we are enabling the receiver, we must wait for the scan to complete.
  if (myScan->enable_rx)
    {
      rx_start =
	pb_inst_radio_shape_cyclops (2, PHASE090, PHASE000, PHASE000, TX_DISABLE,
			     NO_PHASE_RESET, DO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, LONG_DELAY,
			     8, ((myScan->wait_time)*ms/8));
    }

  //If the transmitter is enabled, we start from the TX section of the pulse program.
  if (myScan->enable_tx)
    start = tx_start;
  else if (myScan->enable_rx)
    start = rx_start;
  else
    return RX_AND_TX_DISABLED; 

  // Now wait the repetition delay, then loop back to the beginning. Also reset the phase in anticipation of the next scan
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE, PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, CONTINUE, 0,
 		       myScan->repetitionDelay * 1000.0 * ms);
	
/*****************  END SCAN #0  ***********************************/


/*****************  BEGIN SCAN #1  *************************************/

if (myScan->enable_tx)
    {
      //If blanking is enabled, we must add an additional pulse program interval to compensate for the time the power amplifier needs to "warm up" before we can generate the RF pulse.
      if (myScan->blankingEnable)
	{
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE090, TX_DISABLE,
				 NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1,
				 flagBits, CONTINUE,
				 0, myScan->blankingDelay * ms);

	  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE090, TX_ENABLE,
			       NO_PHASE_RESET, NO_TRIGGER, myScan->use_shape,
			       0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1, flagBits , CONTINUE, 0,
			       myScan->pulseTime * us);
	}
      else
	{

	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE090, TX_ENABLE,
				 NO_PHASE_RESET, NO_TRIGGER,
				 myScan->use_shape, 0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1, 0x00, CONTINUE,
				 0, myScan->pulseTime * us);
	}

      // Output nothing for the transient time.
      pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE090, TX_DISABLE,
			   NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1, 0x00, CONTINUE,
			   0, myScan->transTime * us);
    }

  //If we are enabling the receiver, we must wait for the scan to complete.
  if (myScan->enable_rx)
    {
	pb_inst_radio_shape_cyclops (2, PHASE090, PHASE000, PHASE090, TX_DISABLE,
			     NO_PHASE_RESET, DO_TRIGGER, 0, 0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1, 0x00, LONG_DELAY,
			     8, ((myScan->wait_time)*ms/8));
    }

  //If the transmitter is enabled, we start from the TX section of the pulse program.


  // Now wait the repetition delay, then loop back to the beginning. Also reset the phase in anticipation of the next scan
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE180, TX_DISABLE, PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_1,cycScan->add_sub_im_1,cycScan->swap_1, 0x00, CONTINUE, 0,
		       myScan->repetitionDelay * 1000.0 * ms);

	   
/******************  END SCAN #1  ***********************************************/




/******************  BEGIN SCAN #2  ****************************************************/

if (myScan->enable_tx)
    {
      //If blanking is enabled, we must add an additional pulse program interval to compesate for the time the power amplifier needs to "warm up" before we can generate the RF pulse.
      if (myScan->blankingEnable)
	{
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE180, TX_DISABLE,
				 NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2,
				 flagBits, CONTINUE,0, myScan->blankingDelay * ms);

	  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE180, TX_ENABLE,
			       NO_PHASE_RESET, NO_TRIGGER, myScan->use_shape, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2, 
				   flagBits , CONTINUE, 0, myScan->pulseTime * us);
	}
      else
	{
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE180, TX_ENABLE,
				 NO_PHASE_RESET, NO_TRIGGER,
				 myScan->use_shape, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2, 0x00, CONTINUE,
				 0, myScan->pulseTime * us);
	}

      // Output nothing for the transient time.
      pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE180, TX_DISABLE,
			   NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2, 0x00, CONTINUE,
			   0, myScan->transTime * us);
    }

  //If we are enabling the receiver, we must wait for the scan to complete.
  if (myScan->enable_rx)
    {
	pb_inst_radio_shape_cyclops (2, PHASE090, PHASE000, PHASE180, TX_DISABLE,
			     NO_PHASE_RESET, DO_TRIGGER, 0, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2, 0x00, LONG_DELAY,
			     8, ((myScan->wait_time)*ms/8));
    }

  // Now wait the repetition delay, then loop back to the beginning. Also reset the phase in anticipation of the next scan
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_2,cycScan->add_sub_im_2,cycScan->swap_2, 0x00, CONTINUE, 0,
		       myScan->repetitionDelay * 1000.0 * ms);


/**********************  END SCAN #2  *******************************/



/**********************  BEGIN SCAN #3  ***********************************/

if (myScan->enable_tx)
{
      //If blanking is enabled, we must add an additional pulse program interval to compesate for the time the power amplifier needs to "warm up" before we can generate the RF pulse.
    if (myScan->blankingEnable)
	{
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE,
				 NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3,
				 flagBits, CONTINUE,0, myScan->blankingDelay * ms);

	  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_ENABLE,
			       NO_PHASE_RESET, NO_TRIGGER, myScan->use_shape,
			       0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, flagBits , CONTINUE, 0,
			       myScan->pulseTime * us);
	}
    else
	{
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_ENABLE,
				 NO_PHASE_RESET, NO_TRIGGER,
				 myScan->use_shape, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, CONTINUE,
				 0, myScan->pulseTime * us);
	}

      // Output nothing for the transient time.
      pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE,
			   NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, CONTINUE,
			   0, myScan->transTime * us);
}

  //If we are enabling the receiver, we must wait for the scan to complete.
  if (myScan->enable_rx)
  {
	pb_inst_radio_shape_cyclops (2, PHASE090, PHASE000, PHASE270, TX_DISABLE,
			     NO_PHASE_RESET, DO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, LONG_DELAY,
			     8, ((myScan->wait_time)*ms/8));
  }

  // Now wait the repetition delay, then loop back to the beginning. Also reset the phase in anticipation of the next scan
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, END_LOOP, start,
		       myScan->repetitionDelay * 1000.0 * ms);

/****************  END SCAN #3  *************************************/


			   
  // Stop execution of program.
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, NO_PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, CONTINUE, 0, 1.0 * us);
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, NO_PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, STOP, 0, 1.0 * us);

  pb_stop_programming ();
}
else		//If the number of scan is set to 1
{
	//If we have the transmitter enabled, we must include the pulse program to generate the RF pulse.
  if (myScan->enable_tx)
    {
      // Reset phase initially, so that the phase of the excitation pulse will be the same for every scan.
      if(pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE, PHASE_RESET,
			   NO_TRIGGER, myScan->use_shape, myScan->amplitude,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0,
			   0x00, CONTINUE, 0, 1.0 * us) < 0)
		{
            if (myScan->verbose)
			printf("Your board does not support CYCLOPS\n\n");
			return -11;
		}
/********************  BEGIN SCAN #0  **********************************************/
      //If blanking is enabled, we must add an additional pulse program interval to compesate for the time the power amplifier needs to "warm up" before we can generate the RF pulse.
      if (myScan->blankingEnable)
	{
	  tx_start =
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE,
				 NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0,
				 flagBits, CONTINUE, 0
				 , myScan->blankingDelay * ms);

	  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_ENABLE,
			       NO_PHASE_RESET, NO_TRIGGER, myScan->use_shape,
			       0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, flagBits , CONTINUE, 0,
			       myScan->pulseTime * us);
	}
      else
	{
	  tx_start =
	    pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_ENABLE,
				 NO_PHASE_RESET, NO_TRIGGER,
				 myScan->use_shape, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, CONTINUE,
				 0, myScan->pulseTime * us);
	}

      // Output nothing for the transient time.
      pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE,
			   NO_PHASE_RESET, NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, CONTINUE,
			   0, myScan->transTime * us);
    }

  //If we are enabling the receiver, we must wait for the scan to complete.
  if (myScan->enable_rx)
    {
      rx_start =
	pb_inst_radio_shape_cyclops (2, PHASE090, PHASE000, PHASE000, TX_DISABLE,
			     NO_PHASE_RESET, DO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, LONG_DELAY,
			     8, ((myScan->wait_time)*ms/8));
    }

  //If the transmitter is enabled, we start from the TX section of the pulse program.
  if (myScan->enable_tx)
    start = tx_start;
  else if (myScan->enable_rx)
    start = rx_start;
  else
    return RX_AND_TX_DISABLED; 

  // Now wait the repetition delay, then loop back to the beginning. Also reset the phase in anticipation of the next scan
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE000, TX_DISABLE, PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_0,cycScan->add_sub_im_0,cycScan->swap_0, 0x00, CONTINUE, 0,
 		       myScan->repetitionDelay * 1000.0 * ms);
	
/*****************  END SCAN #0  ***********************************/
  // Stop execution of program.
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, NO_PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, CONTINUE, 0, 1.0 * us);
  pb_inst_radio_shape_cyclops (1, PHASE090, PHASE000, PHASE270, TX_DISABLE, NO_PHASE_RESET,
		       NO_TRIGGER, 0, 0,cycScan->add_sub_re_3,cycScan->add_sub_im_3,cycScan->swap_3, 0x00, STOP, 0, 1.0 * us);

  pb_stop_programming ();
	
}
  return 0;
}

double
checkUndersampling (SCANPARAMS * myScan, int verbose)
{
  int folding_constant;
  double folded_frequency;
  double adc_frequency = myScan->adcFrequency;
  double spectrometer_frequency = myScan->spectrometerFrequency;
  double nyquist_frequency = adc_frequency / 2.0;

  if (spectrometer_frequency > nyquist_frequency)
    {
      if (((spectrometer_frequency / adc_frequency) -
	   (int) (spectrometer_frequency / adc_frequency)) >= 0.5)
	folding_constant =
	  (int) ceil (spectrometer_frequency / adc_frequency);
      else
	folding_constant =
	  (int) floor (spectrometer_frequency / adc_frequency);

      folded_frequency =
	fabs (spectrometer_frequency -
	      ((double) folding_constant) * adc_frequency);

      if (verbose)
	printf
	  ("Undersampling Detected: Spectrometer Frequency (%.4lf MHz) is greater than Nyquist (%.4lf MHz).\n",
	   spectrometer_frequency, nyquist_frequency);

      spectrometer_frequency = folded_frequency;
    }

  return spectrometer_frequency;
}

void mexFunction(int nlhs, mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
    
    int num_points;
    double SF;
    double SW;
    double pulse_time;
    double trans_time;
    double repetition_delay;
    int num_scans;
    double tx_phase;
    int bypass_fir;
    double adc_freq;
    float amp_val;
    int blankingBit;
    double blanking_delay;
    char *shape;
    int shapelen;
    int boardNum;
    int *errorCYCLOPS;
    double *totalTime;
    
    num_points = mxGetScalar(prhs[0]);
    SF = mxGetScalar(prhs[1]);
    SW = mxGetScalar(prhs[2]);
    pulse_time = mxGetScalar(prhs[3]);
    trans_time = mxGetScalar(prhs[4]);
    repetition_delay = mxGetScalar(prhs[5]);
    num_scans = mxGetScalar(prhs[6]);
    tx_phase = mxGetScalar(prhs[7]);
       
    bypass_fir = mxGetScalar(prhs[8]);
    adc_freq = mxGetScalar(prhs[9]);
    amp_val = mxGetScalar(prhs[10]);
    
    shapelen = (mxGetM(prhs[11]) * mxGetN(prhs[11])) + 1;

    
    shape = mxCalloc(shapelen, sizeof(char));
    mxGetString(prhs[11],shape,shapelen);
    
    if ((strcmp(shape,"Hard")!=0)&&(strcmp(shape,"hard")!=0)
    &&(strcmp(shape,"Sinc")!=0)&&(strcmp(shape,"sinc")!=0)) 
    {
    mexErrMsgTxt("Shape must be specified as being 'hard' or 'sinc'");
    }
    blankingBit = mxGetScalar(prhs[12]);
    blanking_delay = (mxGetScalar(prhs[13]))/1000.0; //blanking_delay expected in ms not us.
    boardNum = mxGetScalar(prhs[14]);
    
    plhs[0] = mxCreateNumericMatrix(1,1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(1,1, mxDOUBLE_CLASS, mxREAL);
    errorCYCLOPS=(int*)mxGetPr(plhs[0]);
    totalTime = (double*)mxGetPr(plhs[1]);
    main(boardNum, num_points, SF, SW, pulse_time, trans_time, repetition_delay,
    num_scans, tx_phase, bypass_fir, adc_freq, amp_val, shape, blankingBit, 
    blanking_delay, errorCYCLOPS, (double*)totalTime);
}
