/************************************************************************** 
 * This function was designed for use in conjunction with an NMR Scan 
 * program that doesn't save the data itself.  The current setup allows the
 * user to download and view data at any time - even while the scan is
 * still running. 
 *
 *Function call is as follows:
 *[Real,Imag,status]=pbGetDataPlus(num_points,SF,SW,'fname',num_scans,adc_freq);
 *
 *************************************************************************/

#include "math.h"
#include "mex.h"
#include "spinapi.h"
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "pbGetDataPlus.h"
#define RAM_SIZE (16*1024)

void pbGetDataPlus(int *real, int *imag,
int num_points, double SF, double SW, char *fname,
double pulse_time, double trans_time, double repetition_delay, double tx_phase, int bypass_fir, float amp_val, char *shape,
int num_scans, double adc_freq, int* status, double* res_Freq)
{
    SCANPARAMS *myScan;
    
    myScan = (SCANPARAMS *) malloc (sizeof (SCANPARAMS));
    memset ((void *) myScan, 0, sizeof (SCANPARAMS));
    char txt_fname[1024];
    char fid_fname[1024];
	char jcamp_fname[1024];
	char title_string[412];
	char buff_string[40];
	double resFreq;
    int cmd = 0;
    //SW=SW*1000;
  myScan->nPoints = num_points;
  myScan->spectrometerFrequency = SF;
  myScan->spectralWidth = SW;
  myScan->pulseTime = pulse_time;
  myScan->transTime = trans_time;
  myScan->repetitionDelay = repetition_delay;
  myScan->nScans = num_scans;
  myScan->tx_phase = tx_phase;
  myScan->bypass_fir = bypass_fir;
  myScan->adcFrequency = adc_freq;
  myScan->amplitude = amp_val;
  if ((strcmp(shape,"Sinc")==0) || (strcmp(shape,"sinc")==0))
    {
        myScan->use_shape = 1;
    }
    else myScan->use_shape = 0; 
    
    strncpy(txt_fname, fname, 1024);
    strncat(txt_fname, ".txt", 1024);
    strncpy(fid_fname, fname, 1024);
    strncat(fid_fname, ".fid", 1024);
	strncpy(jcamp_fname, fname, 1024);
    strncat(jcamp_fname, ".dx", 1024);
    SW = SW*1000;       // Converts SW in KHz to MHz
    pb_init();
    
	status[0] = pb_get_data(num_points, real, imag);
    
    // Write the data to an ASCII file
	pb_write_ascii_verbose(txt_fname, num_points, SW, SF, real, imag);
    

	// Write the data to a JCAMP-DX file
	pb_write_jcamp(jcamp_fname, num_points, SW, SF, real, imag);

    /*
//Create Title Block String
      strcat (title_string,"\r\n\r\nPulse Time = ");
      sprintf(buff_string,"%f",myScan->pulseTime);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nTransient Time = ");
      sprintf(buff_string,"%f",myScan->transTime);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nRepetition Delay = ");
      sprintf(buff_string,"%f",myScan->repetitionDelay);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\n# of Scans = ");
      sprintf(buff_string,"%d",myScan->nScans);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nADC Freq = ");
      sprintf(buff_string,"%f",myScan->adcFrequency);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nAmplitude = ");
      sprintf(buff_string,"%f",myScan->amplitude);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\n# of Points = ");
      sprintf(buff_string,"%d",myScan->nPoints);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nSpectral Width = ");
      sprintf(buff_string,"%f",myScan->spectralWidth);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nSpectrometer Freq = ");
      sprintf(buff_string,"%f",myScan->spectrometerFrequency);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nBypass FIR = ");
      sprintf(buff_string,"%d",myScan->bypass_fir);
      strcat (title_string,buff_string);
      strcat (title_string,"\r\nUse Shape = ");
      sprintf(buff_string,"%d",myScan->use_shape);
      strcat (title_string,buff_string);
     
	
    // Write the data to a Felix file
    if(pb_write_felix(fid_fname,title_string, num_points, SW, SF, real, imag) < 0)
    {
		printf("ERROR: Writing fellix using pb_write_felix failed. Check the version of spinapi\n");
		pb_close();
		return; 
	}
     
    
    
    resFreq = pb_fft_find_resonance(num_points, SF*1e6, SW, real, imag);
    res_Freq[0] = resFreq;
     */
     
    pb_close();
    
    return;
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
    
    
    int num_points;
    double SF;
    double SW;
    char *fname;
    int num_scans;
    int *real;
    int *imag;
    int fnamelen;
    double adc_freq;
    int *status;
    int bypass_fir;
    double *res_Freq;
    double pulse_time;
    double trans_time;
    double repetition_delay;
    double tx_phase;
    float amp_val;
    char *shape;
    int shapelen;
    
    
    
    num_points = mxGetScalar(prhs[0]);
    SF = mxGetScalar(prhs[1]);
    SW = mxGetScalar(prhs[2]);
    
    fnamelen = (mxGetM(prhs[3]) * mxGetN(prhs[3])) + 1;
    fname = mxCalloc(fnamelen, sizeof(char));
    mxGetString(prhs[3],fname,fnamelen);
    
    num_scans = mxGetScalar(prhs[4]);
    adc_freq = mxGetScalar(prhs[5]);
    
    
    /*
    pulse_time = mxGetScalar(prhs[6]);
    trans_time = mxGetScalar(prhs[7]);
    repetition_delay = mxGetScalar(prhs[8]);
    tx_phase = mxGetScalar(prhs[9]);
    bypass_fir = mxGetScalar(prhs[10]);
    amp_val = mxGetScalar(prhs[11]);
    shape = mxCalloc(shapelen, sizeof(char));
    mxGetString(prhs[12],shape,shapelen);
    
    
    
    if ((strcmp(shape,"Hard")!=0)&&(strcmp(shape,"hard")!=0)
    &&(strcmp(shape,"Sinc")!=0)&&(strcmp(shape,"sinc")!=0)) 
    {
    mexErrMsgTxt("Shape must be specified as being 'hard' or 'sinc'");
    }
     */
    
    plhs[0] = mxCreateNumericMatrix(num_points,1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(num_points,1, mxINT32_CLASS, mxREAL);
    plhs[2] = mxCreateNumericMatrix(1,1, mxINT32_CLASS, mxREAL);
    //plhs[3] = mxCreateNumericMatrix(1,1, mxDOUBLE_CLASS, mxREAL);
    
    real = (int*)mxGetPr(plhs[0]);
    imag = (int*)mxGetPr(plhs[1]);
    status = (int*)mxGetPr(plhs[2]);
    //res_Freq = (double*)mxGetPr(plhs[3]);
    
    
    pbGetDataPlus((int*)real, (int*)imag, num_points, SF, SW, fname, 
	pulse_time, trans_time, repetition_delay, tx_phase, bypass_fir, amp_val, shape,
	num_scans, adc_freq, (int*)status, (double*)res_Freq);
    
    
}
